declare class FastBooleanArray {
    size: number;
    private buffer;
    constructor(size: number);
    /**
     * Sets a boolean value at the specified index.
     * @param {number} index - The index to set the boolean value at.
     * @param {number} value - The boolean value to set the `index`.
     * @returns {boolean} The boolean value that was set.
     */
    set(index: number, value: boolean): boolean;
    /**
     * like `set` but throws if the index is out of bounds (less than 0 or greater than or equal to the array size).
     * @param {number} index - The index to set the boolean value at.
     * @param {number} value - The boolean value to set the `index`.
     * @returns {boolean} The boolean value that was set.
     * @throws {RangeError} If the index is out of bounds (less than 0 or greater than or equal to the array size).
     */
    setSafe(index: number, value: boolean): boolean;
    /**
     * Sets all bits to the specified boolean value.
     * @param {boolean} value - The boolean value to set all bits to.
     */
    setAll(value: boolean): void;
    /**
     * Gets a boolean value at the specified index.
     * @param {number} index - The index to get the boolean value of.
     * @returns {boolean} The boolean value that was set.
     * @throws {RangeError} If the index is out of bounds (less than 0 or greater than or equal to the array size).
     */
    get(index: number): boolean;
    /**
     * like `get` but throws if the index is out of bounds (less than 0 or greater than or equal to the array size).
     * @param {number} index - The index to get the boolean value of.
     * @returns {boolean} The boolean value that was set.
     * @throws {RangeError} If the index is out of bounds (less than 0 or greater than or equal to the array size).
     */
    getSafe(index: number): boolean;
    /**
     * Resizes the array to a new size, preserving existing data.
     * @param {number} newSize - The new size of the array.
     */
    resize(newSize: number): void;
    get length(): number;
    /**
     * Compares this FastBooleanArray with another for equality.
     * @param {FastBooleanArray} other - The other FastBooleanArray to compare.
     * @returns {boolean} True if both arrays are equal, false otherwise.
     */
    equals(other: FastBooleanArray): boolean;
    /**
     * Makes the array iterable using `for...of`.
     */
    [Symbol.iterator](): Generator<boolean, void, unknown>;
    /**
     * Converts to a standard JavaScript array.
     */
    toArray(): any[];
    /**
     * Returns a proxy to the FastBooleanArray instance
     * With this you can access indexes like on an actual array.
     */
    accessLikeArray(): this;
    /**
     * Applies a callback function to each element in the array.
     * @param {Function} callback - The function to execute on each element.
     */
    forEach(callback: (value: boolean, index: number, array: FastBooleanArray) => void): void;
    /**
     * Creates a new array with the results of calling a provided function on every element.
     * @param {Function} callback - The function to execute on each element.
     */
    map(callback: (value: boolean, index: number, array: FastBooleanArray) => never): any[];
    /**
     * Creates a new array with all elements that pass the test implemented by the provided function.
     * @param {Function} callback - The function to test each element.
     */
    filter(callback: (value: boolean, index: number, array: FastBooleanArray) => boolean): boolean[];
    /**
     * Tests whether at least one element in the array passes the test implemented by the provided function.
     * @param {Function} callback - The function to test each element.
     */
    some(callback: (value: boolean, index: number, array: FastBooleanArray) => boolean): boolean;
    /**
     * Tests whether all elements in the array pass the test implemented by the provided function.
     * @param {Function} callback - The function to test each element.
     */
    every(callback: (value: boolean, index: number, array: FastBooleanArray) => boolean): boolean;
    /**
     * Reduces the array to a single value by applying a function to each element.
     * @param {Function} callback - The function to execute on each element.
     * @param {any} initialValue - The initial value for the accumulator.
     */
    reduce<T>(callback: (accumulator: T, value: boolean, index: number, array: FastBooleanArray) => T, initialValue: T): T;
    /**
     * Generates a string where every boolean is either a 0 or 1.
     */
    toString(): string;
    /**
     * Returns a FastBooleanArray from .toString() output
     */
    static fromString(value: string): FastBooleanArray;
    /**
     * Returns a FastBooleanArray from a boolean array or number array
     */
    static fromArray(value: boolean[] | number[]): FastBooleanArray;
    /**
     * Returns a FastBooleanArray from a string, boolean array or number array
     */
    static from(value: string | boolean[] | number[]): FastBooleanArray;
}

export { FastBooleanArray as default };
